package com.yizhi.application.feign;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.yizhi.application.vo.VideoInfoVo;
import com.yizhi.application.vo.VideoPlayInfoVo;
import com.yizhi.application.vo.VideoUploadAddressAuthVo;

import cc.mrbird.febs.common.core.response.BizResponse;
import io.swagger.annotations.ApiOperation;

import static com.yizhi.core.application.constant.FebsServerConstant.CLOUD_COMMUNITY_ALIYUN;

@FeignClient(name = CLOUD_COMMUNITY_ALIYUN, contextId = "AliyunVideoClient", path = "/api/video")
public interface AliyunVideoClient {

	@ApiOperation("获取视频上传地址和凭证")
    @GetMapping("/video/address/auth/get")
    @ResponseBody
    public BizResponse<VideoUploadAddressAuthVo> getVideoUploadAddressAuth(@RequestParam("title")String title
    		, @RequestParam("fileName")String fileName) throws ServerException, ClientException;
    
    @ApiOperation("刷新视频上传凭证")
    @GetMapping("/video/address/auth/refresh/get")
    @ResponseBody
    public BizResponse<VideoUploadAddressAuthVo> refreshVideoUploadAddressAuth(@RequestParam("videoId")String videoId) throws ServerException, ClientException;
    
    @ApiOperation("获取视频播放地址")
    @GetMapping("/play/url/get")
    public BizResponse<VideoPlayInfoVo> getPlayUrl(@RequestParam("videoId")String videoId) throws ServerException, ClientException;
    
    @ApiOperation("获取视频播放凭证")
    @GetMapping("/play/auth/get")
    public BizResponse<String> getPlayAuth(@RequestParam("videoId")String videoId) throws ServerException, ClientException;
    
    @ApiOperation("获取视频信息")
    @GetMapping("/info/get")
    public BizResponse<VideoInfoVo> getVideoInfo(@RequestParam("videoId")String videoId) throws ServerException, ClientException;
}
