package com.yizhi.application.controller;


import java.net.URL;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.aliyun.oss.ClientException;
import com.aliyun.oss.HttpMethod;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.GeneratePresignedUrlRequest;
import com.yizhi.application.service.VideoInfoService;
import com.yizhi.application.util.PropertiesUtil;
import com.yizhi.application.vo.ImageUploadAddressAuthVo;

import cc.mrbird.febs.common.core.response.BizResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@RequestMapping(value = "/manage/image")
@Api(tags = "管理端-阿里云-图片相关接口")
@PreAuthorize("#oauth2.hasScope('manage')")
public class ManageImageController {

    @Autowired
    PropertiesUtil propertiesUtil;
    
    @Autowired
    private VideoInfoService videoInfoService;

    @ApiOperation(value = "图片自定义处理", notes = "图片自定义处理", response = BizResponse.class)
    @RequestMapping(value = "/style", method = RequestMethod.GET)
    public BizResponse<String> style(
            @ApiParam(name = "key", value = "图片在oss存储路径", required = true) @RequestParam(value = "key", required = true) String key,
            @ApiParam(name = "styleName", value = "自定义样式名", required = true) @RequestParam(value = "styleName", required = true) String styleName
    ) {
        String endpoint = propertiesUtil.getPublicParam().get("endpoint");
        String accessKeyId = propertiesUtil.getPublicParam().get("accessKeyId");
        String accessKeySecret = propertiesUtil.getPublicParam().get("accessKeySecret");
        OSSClient ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        String bucketName = propertiesUtil.getPublicParam().get("bucket");

        try {

            // 图片处理样式
            String style = "style/" + styleName;
            // 过期时间10分钟
            java.util.Date expiration = new java.util.Date(new java.util.Date().getTime() + 1000 * 60 * 10);
            GeneratePresignedUrlRequest req = new GeneratePresignedUrlRequest(bucketName, key, HttpMethod.GET);
            req.setExpiration(expiration);
            req.setProcess(style);
            URL signedUrl = ossClient.generatePresignedUrl(req);
            System.out.println(signedUrl);
            return BizResponse.ok(signedUrl.toString());
        } catch (OSSException oe) {
            System.out.println("Caught an OSSException, which means your request made it to OSS, "
                    + "but was rejected with an error response for some reason.");
            System.out.println("Error Message: " + oe.getErrorCode());
            System.out.println("Error Code:       " + oe.getErrorCode());
            System.out.println("Request ID:      " + oe.getRequestId());
            System.out.println("Host ID:           " + oe.getHostId());
            return BizResponse.fail(oe.getMessage());
        } catch (ClientException ce) {
            System.out.println("Caught an ClientException, which means the client encountered "
                    + "a serious internal problem while trying to communicate with OSS, "
                    + "such as not being able to access the network.");
            System.out.println("Error Message: " + ce.getMessage());
            return BizResponse.fail(ce.getMessage());
        } catch (Throwable e) {
            e.printStackTrace();
            return BizResponse.fail(e.getMessage());
        } finally {
            ossClient.shutdown();
        }
    }
    
    @ApiOperation("获取图片上传地址和凭证")
    @GetMapping("/address/auth/get")
    public BizResponse<ImageUploadAddressAuthVo> getPlayAuth() throws ClientException, com.aliyuncs.exceptions.ClientException{
    	return BizResponse.ok(videoInfoService.getImageUploadAddressAuth());
    }
}
