package com.yizhi.application.controller.student;

import java.sql.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.aliyun.oss.OSSClient;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.model.MatchMode;
import com.aliyun.oss.model.PolicyConditions;
import com.yizhi.application.util.PropertiesUtil;

import cc.mrbird.febs.common.core.response.BizResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;

@Slf4j
@RestController
@Api(tags = "学员端 policy相关")
@RequestMapping(value = "/student/upload")
//@PreAuthorize("#oauth2.hasScope('student')")
public class StudentPolicyController {

    @Value("${ACTIVE}")
    public String active;

    @Autowired
    PropertiesUtil propertiesUtil;

    @ApiOperation(value = "web端上传签名获得", notes = "web端上传签名获得", response = BizResponse.class)
    @RequestMapping(value = "/policy", method = RequestMethod.GET)
    public BizResponse<JSONObject> token(@RequestParam(name = "needTranscode", required = false) Boolean needTranscode, @RequestParam(name = "type", required = false, defaultValue = "0") Integer type) {
        String endpoint = propertiesUtil.getPublicParam().get("endpoint");
        String accessId = propertiesUtil.getPublicParam().get("accessKeyId");
        String accessKey = propertiesUtil.getPublicParam().get("accessKeySecret");
        String dir = propertiesUtil.getPublicParam().get("ossdir");
        String bucket = null;

        if (needTranscode == null || !needTranscode) {
            bucket = propertiesUtil.getPublicParam().get("bucket");
        } else {
            bucket = propertiesUtil.getPublicParam().get("transcodeBucket");
            switch (type) {
                //作业本
                case 0:
                    dir = "exerciseBook";
                    break;
                //资料
                case 1:
                    dir = "document";
                    break;
                //案例
                case 2:
                    dir = "studentCase";
                    break;
                //论坛
                case 3:
                    dir = "forum";
                    break;
            }
        }

        String host = "https://" + bucket + "." + endpoint;


        OSSClient client = new OSSClient(endpoint, accessId, accessKey);
        try {

            long expireTime = 60 * 24;
            long expireEndTime = System.currentTimeMillis() + expireTime * 1000;
            Date expiration = new Date(expireEndTime);
            PolicyConditions policyConds = new PolicyConditions();
            policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 10485760000L);
            policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY, dir);

            String postPolicy = client.generatePostPolicy(expiration, policyConds);
            byte[] binaryData = postPolicy.getBytes("utf-8");
            String encodedPolicy = BinaryUtil.toBase64String(binaryData);
            String postSignature = client.calculatePostSignature(postPolicy);


            Map<String, String> respMap = new LinkedHashMap<String, String>();
            respMap.put("accessid", accessId);
            respMap.put("policy", encodedPolicy);
            respMap.put("signature", postSignature);
            //respMap.put("expire", formatISO8601Date(expiration));
            respMap.put("dir", dir);
            respMap.put("host", host);
            respMap.put("expire", String.valueOf(expireEndTime / 1000));
            respMap.put("active", active);
            JSONObject ja1 = JSONObject.fromObject(respMap);
            log.info(ja1.toString());
            return BizResponse.ok(ja1);
        } catch (Exception e) {
            return BizResponse.fail(e.getMessage());
        } finally {
            client.shutdown();
        }

    }

}
