package com.yizhi.application.controller.student;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.yizhi.application.service.VideoInfoService;
import com.yizhi.application.vo.VideoPlayInfoVo;
import com.yizhi.application.vo.VideoUploadAddressAuthVo;

import cc.mrbird.febs.common.core.response.BizResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value = "/student/video")
@Api(tags = "学员端-阿里云-视频相关接口")	
public class StudentVideoController {

    @Autowired
    private VideoInfoService videoInfoService;

    @ApiOperation("获取视频上传地址和凭证")
    @GetMapping("/address/auth/get")
    @ResponseBody
    public BizResponse<VideoUploadAddressAuthVo> getVideoUploadAddressAuth(@RequestParam("title")String title
    		, @RequestParam("fileName")String fileName) throws ServerException, ClientException{
    	return BizResponse.ok(videoInfoService.getVideoUploadAddressAuth(title, fileName));
    }
    
    @ApiOperation("刷新视频上传凭证")
    @GetMapping("/address/auth/refresh/get")
    @ResponseBody
    public BizResponse<VideoUploadAddressAuthVo> refreshVideoUploadAddressAuth(@RequestParam("videoId")String videoId) throws ServerException, ClientException{
    	return BizResponse.ok(videoInfoService.refreshVideoUploadAddressAuth(videoId));
    }
    
    @ApiOperation("获取视频播放地址")
    @GetMapping("/play/url/get")
    public BizResponse<VideoPlayInfoVo> getPlayUrl(@RequestParam("videoId")String videoId) throws ServerException, ClientException{
    	return BizResponse.ok(videoInfoService.getPlayUrl(videoId));
    }
    
    @ApiOperation("获取视频播放凭证")
    @GetMapping("/play/auth/get")
    public BizResponse<String> getPlayAuth(@RequestParam("videoId")String videoId) throws ServerException, ClientException{
    	return BizResponse.ok(videoInfoService.getPlayAuth(videoId));
    }
    
}
