package com.yizhi.application.mq;


import com.yizhi.application.domain.BizMessage;
import com.yizhi.application.service.IAliyunBizMessageService;
import com.yizhi.core.application.id.IdGenerator;
import com.yizhi.core.application.mq.AliyunBizMessage;
import com.yizhi.core.application.mq.EventQueue;
import com.yizhi.core.application.mq.EventWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

@Component
@RabbitListener(queues = EventQueue.BIZ_MESSAGE)
@Slf4j
@RequiredArgsConstructor
public class BizMessageListener {
    private final IAliyunBizMessageService bizMessageService;
    private final IdGenerator generator;


    @RabbitHandler
    public void processBizEvent(EventWrapper<AliyunBizMessage> eventWrapper) {
        if (eventWrapper == null || eventWrapper.getData() == null) {
            log.error("接收到来自queue:" + EventQueue.BIZ_MESSAGE + "的消息，但eventWrapper参数异常");
            return;
        }
        AliyunBizMessage bizMessage = eventWrapper.getData();
        if (bizMessage.getBizId() == null || bizMessage.getBizType() == null || bizMessage.getEvenType() == null) {
            log.error("接收到来自queue:" + EventQueue.BIZ_MESSAGE + "的消息，但bizMessage参数异常");
            return;
        }
        //先入库,后续等待定时任务处理
        saveBizMessage(bizMessage);
    }

    /**
     * 消息入库
     *
     * @param bizMessage
     */
    private void saveBizMessage(AliyunBizMessage bizMessage) {
        BizMessage message = new BizMessage();
        message.setId(generator.generate());
        message.setBizId(bizMessage.getBizId());
        message.setBizType(bizMessage.getBizType());
        message.setEventType(bizMessage.getEvenType().getType());
        message.setAliyunId(bizMessage.getAliyunBizId());
        bizMessageService.save(message);
    }
}
