package com.yizhi.application.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.vod.model.v20170321.CreateUploadImageRequest;
import com.aliyuncs.vod.model.v20170321.CreateUploadImageResponse;
import com.aliyuncs.vod.model.v20170321.CreateUploadVideoRequest;
import com.aliyuncs.vod.model.v20170321.CreateUploadVideoResponse;
import com.aliyuncs.vod.model.v20170321.GetPlayInfoResponse;
import com.aliyuncs.vod.model.v20170321.GetVideoInfoRequest;
import com.aliyuncs.vod.model.v20170321.GetVideoInfoResponse;
import com.aliyuncs.vod.model.v20170321.GetVideoPlayAuthRequest;
import com.aliyuncs.vod.model.v20170321.GetVideoPlayAuthResponse;
import com.aliyuncs.vod.model.v20170321.RefreshUploadVideoRequest;
import com.aliyuncs.vod.model.v20170321.RefreshUploadVideoResponse;
import com.yizhi.application.service.VideoInfoService;
import com.yizhi.application.util.PropertiesUtil;
import com.yizhi.application.vo.ImageUploadAddressAuthVo;
import com.yizhi.application.vo.PlayInfoVo;
import com.yizhi.application.vo.VideoBaseVo;
import com.yizhi.application.vo.VideoInfoVo;
import com.yizhi.application.vo.VideoPlayInfoVo;
import com.yizhi.application.vo.VideoUploadAddressAuthVo;
import com.yizhi.core.application.convert.DomainConverter;

@Service
public class VideoInfoServiceImpl implements VideoInfoService {

	@Autowired
    private DefaultAcsClient acsClient;
	
	@Autowired
    PropertiesUtil propertiesUtil;
	
	@Override
	public VideoUploadAddressAuthVo getVideoUploadAddressAuth(String title, String fileName) throws ServerException, ClientException {
		CreateUploadVideoRequest request = new CreateUploadVideoRequest();
	    request.setTitle(title);
	    request.setFileName(fileName);
	    request.setTemplateGroupId(propertiesUtil.getPublicParam().get("templateGroupId"));
	    CreateUploadVideoResponse response = acsClient.getAcsResponse(request);
	    return VideoUploadAddressAuthVo.builder()
	    		.videoId(response.getVideoId())
	    		.uploadAddress(response.getUploadAddress())
	    		.uploadAuth(response.getUploadAuth())
	    		.build();
	}

	@Override
	public VideoUploadAddressAuthVo refreshVideoUploadAddressAuth(String videoId) throws ServerException, ClientException {
		RefreshUploadVideoRequest request = new RefreshUploadVideoRequest();
	    request.setVideoId(videoId);
	    RefreshUploadVideoResponse response = acsClient.getAcsResponse(request);
		return VideoUploadAddressAuthVo.builder()
	    		.uploadAddress(response.getUploadAddress())
	    		.uploadAuth(response.getUploadAuth())
	    		.build();
	}

	@SuppressWarnings("unchecked")
	@Override
	public VideoPlayInfoVo getPlayUrl(String videoId) throws ServerException, ClientException {
		com.aliyuncs.vod.model.v20170321.GetPlayInfoRequest request = new com.aliyuncs.vod.model.v20170321.GetPlayInfoRequest();
		request.setVideoId(videoId);
		GetPlayInfoResponse response = acsClient.getAcsResponse(request);
		List<GetPlayInfoResponse.PlayInfo> playInfoList = response.getPlayInfoList();
		GetPlayInfoResponse.VideoBase base = response.getVideoBase();
		List<PlayInfoVo> playInfoVoList = DomainConverter.getInstance().toVOList(playInfoList, PlayInfoVo.class);
		VideoBaseVo baseVo = (VideoBaseVo) DomainConverter.getInstance().toVO(base, VideoBaseVo.class);
		VideoPlayInfoVo result = new VideoPlayInfoVo();
		result.setPlayInfoList(playInfoVoList);
		result.setVideoBase(baseVo);
		return result;
	}

	@Override
	public String getPlayAuth(String videoId) throws ServerException, ClientException {
		GetVideoPlayAuthRequest request = new GetVideoPlayAuthRequest();
		request.setVideoId(videoId);
		GetVideoPlayAuthResponse response = acsClient.getAcsResponse(request);
		return response.getPlayAuth();
	}

	@Override
	public ImageUploadAddressAuthVo getImageUploadAddressAuth() throws ServerException, ClientException {
		CreateUploadImageRequest request = new CreateUploadImageRequest();
	    request.setImageType("default");
	    
	    CreateUploadImageResponse response = acsClient.getAcsResponse(request);
	    
		return ImageUploadAddressAuthVo.builder()
				.uploadAddress(response.getUploadAddress())
				.uploadAuth(response.getUploadAuth())
				.imageId(response.getImageId())
				.imageUrl(response.getImageURL())
				.build();
	}

	@SuppressWarnings("unchecked")
	@Override
	public VideoInfoVo getVideoInfo(String videoId) throws ServerException, ClientException {
		GetVideoInfoRequest request = new GetVideoInfoRequest();
	    request.setVideoId(videoId);
	    GetVideoInfoResponse response = acsClient.getAcsResponse(request);
		return (VideoInfoVo) DomainConverter.getInstance().toVO(response.getVideo(), VideoInfoVo.class);
	}

}
